<?php

namespace Drupal\pwbi;

use Drupal\oauth2_client\Entity\Oauth2Client;
use Drupal\oauth2_client\Service\CredentialProvider;

/**
 * Decorator for CredentialProvider to get the tenant.
 */
class TenantCredentialProvider extends CredentialProvider {

  /**
   * Get the tenant configured in the credentials form.
   *
   * @param string $pluginId
   *   The id of the authentication plugin.
   *
   * @return mixed
   *   The tenant id.
   */
  public function getTenant(string $pluginId): mixed {
    /** @var \Drupal\oauth2_client\Entity\Oauth2Client $oauthClientConfig */
    $oauthClientConfig = $this->entityTypeManager->getStorage('oauth2_client')->load($pluginId);
    if (!($oauthClientConfig instanceof Oauth2Client)) {
      throw new \Exception(sprintf('There is no configuration for plugin:%s', $pluginId));
    }
    return $oauthClientConfig->getThirdPartySetting('pwbi', 'tenant');
  }

}
