<?php

declare(strict_types=1);

namespace Drupal\Tests\pwbi\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Test Power Bi authentication.
 */
class AuthTest extends BrowserTestBase {

  protected const TENANT = 'my-tenant';
  protected const CLIENT_ID = 'client-id';
  protected const CLIENT_SECRET = 'client-secret';
  protected const LABEL = 'Test service principal auth';
  protected const PLUGIN_ID = 'pwbi_service_principal';

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'oauth2_client',
    'pwbi',
  ];

  /**
   * Test oauth client plugin.
   */
  public function testAuthenticationPlugin(): void {
    // Test credentials configuration.
    /** @var \Drupal\user\Entity\User $account */
    $account = $this->drupalCreateUser(['administer oauth2 clients']);
    $this->drupalLogin($account);

    $this->drupalGet('admin/config/system/oauth2-client');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('admin/config/system/oauth2-client/' . self::PLUGIN_ID . '/edit');
    $this->assertSession()->statusCodeEquals(200);
    $page = $this->getSession()->getPage();
    $page->fillField('edit-label', self::LABEL);
    $page->fillField('edit-client-id', self::CLIENT_ID);
    $page->fillField('edit-client-secret', self::CLIENT_SECRET);
    $page->fillField('edit-tenant', self::TENANT);
    $page->checkField('edit-status');
    $page->pressButton('edit-submit');
    $this->assertSession()->pageTextContains("Updated oauth2 client " . self::LABEL . ".");
    $this->assertSession()->elementTextContains('css', '.responsive-enabled', 'Enabled');

    // Test authentication provider credentials creation.
    /** @var \Drupal\pwbi\Plugin\Oauth2Client\PwbiServicePrincipal $oauth_service_principal */
    $oauth_service_principal = \Drupal::service('oauth2_client.service')->getClient(self::PLUGIN_ID);
    $oauth_provider = $oauth_service_principal->getProvider();
    $reflectionClass = new \ReflectionClass('League\OAuth2\Client\Provider\GenericProvider');
    $credentials = [
      'clientId' => self::CLIENT_ID,
      'clientSecret' => self::CLIENT_SECRET,
      'urlAuthorize' => sprintf("https://login.microsoftonline.com/%s/oauth2/v2.0/authorize/", self::TENANT),
      'urlAccessToken' => sprintf("https://login.windows.net/%s/oauth2/token/", self::TENANT),
      'urlResourceOwnerDetails' => "https://analysis.windows.net/powerbi/api",
    ];
    foreach ($credentials as $credential => $value) {
      $this->assertEquals($value, $reflectionClass->getProperty($credential)->getValue($oauth_provider));
    }
    $this->assertEquals(self::TENANT, $oauth_service_principal->getTenant());
    $this->assertEquals("https://analysis.windows.net/powerbi/api/.default", $oauth_service_principal->getScope());
  }

}
